;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          audioscrobbler.lisp
;;;; Purpose:       Audioscrobbler/lastfm wrapper
;;;; Programmer:    Nicolas Lamirault <nicolas.lamirault@gmail.com>
;;;;
;;;; This file, part of audioscrobbler, is Copyright (c) 2007 by Nicolas Lamirault
;;;;
;;;; audioscrobbler users are granted the rights to distribute and use this software
;;;; as governed by the terms of the Lisp Lesser GNU Public License
;;;; (http://opensource.franz.com/preamble.html), also known as the LLGPL.
;;;;
;;;; *************************************************************************


(in-package :cl-audioscrobbler)



(defun get-md5sum-code (password challenge)
 "Create a string like md5(md5(PASSWORD) + CHALLENGE)."
 (map 'string #'code-char
      (md5:md5sum-sequence
       (concatenate 'string
                    (map 'string #'code-char (md5:md5sum-sequence password))
                    challenge))))


(defun get-identification (username &optional (client-id +client-id+)
                          (client-version +client-version+))
 "Make an HTTP request to retreive challenge for protocol identification.
Returns 2 values : ID and URL."
 (let ((query (format nil +protocol-identify+
                      client-id client-version username)))
   (multiple-value-bind (body status-code headers uri stream must-close)
       (drakma:http-request (url-rewrite:url-encode query))
     (declare (ignore headers uri stream must-close))
     (when (and status-code (= status-code 200))
       (with-input-from-string (stream body)
         (loop as line = (read-line stream nil)
            until (null line)
            when (cl-ppcre:scan "UPDATE" line)
            return (let ((challenge (read-line stream nil))
                         (url (read-line stream nil)))
                     (values challenge url))))))))


(defun post-song (username password artist album song length time)
 (multiple-value-bind (challenge url)
     (get-identification username)
   (let* ((md5 (get-md5sum-code password challenge))
          (query (format nil (concatenate 'string url "?" +protocol-post+)
                         artist song album
                         length time (get-current-date) username))
          (uri (cl-ppcre:regex-replace-all " "
                                           (url-rewrite:add-get-param-to-url query "s" md5)
                                           "+")))
     (print uri)
     (multiple-value-bind (body status-code headers uri stream must-close)
         (drakma:http-request (url-rewrite:url-encode query))
     (declare (ignore headers uri stream must-close))
     (when (and status-code (= status-code 200))
         (with-input-from-string (stream body)
           (loop as line = (read-line stream nil)
              until (null line)
              do (print line))))))))
