;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          tools.lisp
;;;; Purpose:       Some tools for the Audioscrobbler/lastfm wrapper
;;;; Programmer:    Nicolas Lamirault <nicolas.lamirault@gmail.com>
;;;;
;;;; This file, part of audioscrobbler, is Copyright (c) 2007 by Nicolas Lamirault
;;;;
;;;; audioscrobbler users are granted the rights to distribute and use this software
;;;; as governed by the terms of the Lisp Lesser GNU Public License
;;;; (http://opensource.franz.com/preamble.html), also known as the LLGPL.
;;;;
;;;; *************************************************************************


(in-package :cl-audioscrobbler)


(defun get-current-date ()
 "Get current date in YYYY-MM-DD mm:hh:ss format."
 (multiple-value-bind
       (second minute hour date month year day-of-week dst-p tz)
     (get-decoded-time)
   (declare (ignore day-of-week dst-p tz))
   (format nil "~2,'0d-~2,'0d-~2,'0d ~2,'0d:~2,'0d:~2,'0d"
           year month date hour minute second)))


(defun perform-audioscrobbler (query)
  "Retreive informations from Audioscrobbler Web Service."
  (when *debug*
    (format t "Audioscrobbler query : ~A~%" query))
  (multiple-value-bind (body status-code headers uri stream must-close)
      (drakma:http-request query)
    (declare (ignore headers uri stream must-close))
    (when (and status-code (= status-code 200))
      body)))


(defun get-xml-answer (query)
  "Make a request to the Audioscrobbler Web Service, and parse XML output."
  (let ((content (perform-audioscrobbler query)))
    (when content 
      (s-xml:parse-xml-string content))))


